%%%% Initial baby demo: storing 2 patterns in a 3-neuron reservoir
%%%% Designed using Matlab R2008b
%%%% H. Jaeger, May 17, 2015
%%%%
%%%% Note: users running later versions of Matlab reported that some
%%%% conceptor demos wouldn't succeed. The cause seems to be that my old
%%%% Matlab version uses a different (and better) implementation of the inv
%%%% function than later Matlab versions. Replacing all calls to "inv" by
%%%% calls to "pinv" so far has always resolved the problems. 

addpath('./helpers');


%%% Experiment control
useCannedNetwork = 1; % if this flag is set to 1, the reservoir used 
                      % in the article is run; if set to 0, new random
                      % weights are created.
randstate = 1; % setting the seed index for the random number generators



%%% Setting system params
N = 3; % network size
NetSR = 1.3; % spectral radius of initial reservoir weight matrix
NetinpScaling = 1.4; % scaling of input weights
BiasScaling = 0.1; % size of bias

%%% loading learning
TychonovAlpha = .0; % regularizer for  W training
washoutLength = 100;
learnLength = 200;
signalPlotLength = 15;

%%% pattern readout learning
TychonovAlphaReadout = .0; % regularizer for  Wout training
learnLengthWout = 200;


%%% C learning and testing
alphas = [12, 20]; % apertures of the conceptors
recallTestLength = 400;

%%% Setting patterns
patterns = [1 2]; % which of the patterns defined below are used
% Define pattern nr 1
patts{1} = @(n) 1 * sin(2 * pi * n / 3.1504531) ;
% Define pattern nr 2
period2 = 2;
rawp = randn(1, period2);
maxVal = max(rawp); minVal = min(rawp);
rp = 0.5 * (2 * (rawp - minVal) / (maxVal - minVal) - 1);
patts{2} = @(n) rp(mod(n, period2 ) + 1);


%%% set random seeds. Note: in newer versions of Matlab, this may have to
%%% be re-programmed since Matlab changed its random number syntax
randn('state', randstate);
rand('twister', randstate);

%%% Create reservoir weights
if useCannedNetwork
    load canned3DimDemoNet;    
else
    WstarRaw = generate_internal_weights(N, 1);
    WinRaw = randn(N, 1);
    WbiasRaw = randn(N, 1);   
    
    %%% Scale raw weights     
    Wstar = NetSR * WstarRaw;
    Win = NetinpScaling * WinRaw;
    Wbias = BiasScaling * WbiasRaw;    
end


I = eye(N);

%%% compute readout by driving reservoir with random input and reconstruct
%%% it with readout weights

    xCollector = zeros(N, learnLengthWout);
    pCollector = zeros(1, learnLengthWout);
    x = zeros(N, 1);
    for n = 1:(washoutLength + learnLength)
        u = 1.5 * randn;
        x = tanh(Wstar * x + Win * u + Wbias);
        if n > washoutLength
            xCollector(:, n - washoutLength) = x;
            pCollector(1, n - washoutLength) = u;
        end
    end
    Wout = (inv(xCollector * xCollector' + ...
        TychonovAlphaReadout * eye(N)) * xCollector * pCollector')';
    % compute training error (normalized root mean square error)
    NRMSE_readout = nrmse(Wout*xCollector, pCollector);
    disp(sprintf('readout NRMSE: %g  absWeight: %g', NRMSE_readout, ...
        mean(abs(Wout))));




%%% load patterns

% harvest data from network externally driven by patterns
Np = length(patterns);
allTrainArgs = zeros(N, Np * learnLength);
allTrainOldArgs = zeros(N, Np * learnLength);
allTrainTargs = zeros(N, Np * learnLength);
allTrainOuts = zeros(1, Np * learnLength);
xCollectors = cell(1,Np);
SRCollectors = cell(1,Np);
URCollectors = cell(1,Np);
patternRs = cell(1,Np);
train_xPL = cell(1,Np);
train_pPL = cell(1,Np);
startXs = zeros(N, Np);
% collect data from driving native reservoir with different drivers
for p = 1:Np
    patt = patts{patterns(p)}; % current pattern generator
    xCollector = zeros(N, learnLength );
    xOldCollector = zeros(N, learnLength );
    pCollector = zeros(1, learnLength );
    x = zeros(N, 1);
    for n = 1:(washoutLength + learnLength)
        u = patt(n); % pattern input
        xOld = x;
        x = tanh(Wstar * x + Win * u + Wbias);
        if n > washoutLength
            xCollector(:, n - washoutLength ) = x;
            xOldCollector(:, n - washoutLength ) = xOld;
            pCollector(1, n - washoutLength) = u;
        end
    end
    
    xCollectors{1,p} = xCollector;
    R = xCollector * xCollector' / learnLength;
    [Ux Sx Vx] = svd(R);
    SRCollectors{1,p} = Sx;
    URCollectors{1,p} = Ux;
    patternRs{p} = R;
    
    
    startXs(:,p) = x;
    train_xPL{1,p} = xCollector(:,1:signalPlotLength);
    train_pPL{1,p} = pCollector(1,1:signalPlotLength);
    
    allTrainArgs(:, (p-1)*learnLength+1:p*learnLength) = ...
        xCollector;
    allTrainOldArgs(:, (p-1)*learnLength+1:p*learnLength) = ...
        xOldCollector;
    allTrainOuts(1, (p-1)*learnLength+1:p*learnLength) = ...
        pCollector;
    allTrainTargs(:, (p-1)*learnLength+1:p*learnLength) = ...
        Win * pCollector;
end


Wtargets = (atanh(allTrainArgs) - repmat(Wbias,1,Np*learnLength));
W = (inv(allTrainOldArgs * allTrainOldArgs' + ...
    TychonovAlpha * eye(N)) * allTrainOldArgs * Wtargets')';

% training errors per neuron
NRMSE_W = nrmse(W*allTrainOldArgs, Wtargets);
disp(sprintf('W NRMSE: %g  absSize: %g', mean(NRMSE_W), ...
    mean(mean(abs(W)))));


%%% compute conceptors
Cs = cell(4, Np);
for p = 1:Np
    R = patternRs{p};
    [U S V] = svd(R);
    Snew = (S * inv(S + alphas(p)^(-2) * eye(N)));
    
    C = U * Snew * U';
    Cs{1, p} = C;
    Cs{2, p} = U;
    Cs{3, p} = diag(Snew);
    Cs{4, p} = diag(S);
end

%%% test with C
x_CTestPL = zeros(3, recallTestLength, Np);
p_CTestPL = zeros(1, recallTestLength, Np);
for p = 1:Np
    C = Cs{1, p};
    x = 0.5*randn(N,1);
    for n = 1:recallTestLength + washoutLength
        x = tanh(W *  x + Wbias);
        x = C * x;
        if n > washoutLength
            x_CTestPL(:,n-washoutLength,p) = x(1:3,1);
            p_CTestPL(:,n-washoutLength,p) = Wout * x;
        end
    end
end



%%% plotting

test_pAligned_PL = cell(1,Np);
test_xAligned_PL = cell(1,Np);
NRMSEsAligned = zeros(1,Np);

% phase-align test output with original signal, by interpolating both
% signals, then shift one against the other for optimal fit, then
% subsampling again to original sampling rate
for p = 1:Np
    intRate = 20; % interpolation factor
    thisDriver = train_pPL{1,p};
    thisOut = p_CTestPL(1,:,p);
    thisDriverInt = interp1((1:signalPlotLength)',thisDriver',...
        (1:(1/intRate):signalPlotLength)', 'spline')';
    thisOutInt = interp1((1:recallTestLength)', thisOut',...
        (1:(1/intRate):recallTestLength)', 'spline')';
    
    L = size(thisOutInt,2); M = size(thisDriverInt,2);
    phasematches = zeros(1,L - M);
    for phaseshift = 1:(L - M)
        phasematches(1,phaseshift) = ...
            norm(thisDriverInt - ...
            thisOutInt(1,phaseshift:phaseshift+M-1));
    end
    [maxVal maxInd] = max(-phasematches);
    test_pAligned_PL{1,p} = ...
        thisOutInt(1,maxInd:intRate:...
        (maxInd+intRate*signalPlotLength-1));
    coarseMaxInd = ceil(maxInd / intRate);
    test_xAligned_PL{1,p} = ...
        x_CTestPL(:,coarseMaxInd:coarseMaxInd+signalPlotLength-1,p);
    NRMSEsAligned(1,p) = ...
        nrmse(test_pAligned_PL{1,p},train_pPL{1,p});
end

meanNRMSE = mean(NRMSEsAligned);
disp(sprintf('mean NRMSE: %g', mean(NRMSEsAligned)));


%%% the overview figure
%%
figure(1); clf;
fs = 14; % font size
set(gcf, 'WindowStyle','normal');
set(gcf,'Position', [900 550 800 400]);

for p = 1:Np
    subplot(Np,3,(p-1)*3+1);
    hold on;
    plot(test_pAligned_PL{1,p}, 'LineWidth',6,...
        'Color', 0.8 * [1 1 1]); 
    if p == 1
        thiscol = 'r';
    else
        thiscol = 'b';
    end
    plot(train_pPL{1,p},thiscol,'LineWidth',1.5); 
    hold off;
    if p == 1
        title('p and y','FontSize',fs);
    end
    if p ~= Np
        set(gca, 'XTickLabel',[]);
    end
    set(gca, 'YLim',[-1,1], 'XLim', [0 signalPlotLength], ...
        'Box', 'on','FontSize',fs);
    
    
    subplot(Np,3,(p-1)*3+2); 
    hold on;
    if p == 1
        thiscol = [1 0 0];
    else
        thiscol = [0 0 1];
    end
    plot(train_xPL{1,p}(1,:)','Color', 1 * thiscol,'LineWidth',2);
    plot(train_xPL{1,p}(2,:)','Color', .7 * thiscol,'LineWidth',2);
    plot(train_xPL{1,p}(3,:)','Color', .4 * thiscol,'LineWidth',2);
    hold off;
    if p == 1
        title('state pattern','FontSize',fs);
    end
    if p ~= Np
        set(gca, 'XTickLabel',[]);
    end
    set(gca,'YLim',[-1,1], 'FontSize',fs, 'XLim', [0 signalPlotLength], 'Box', 'on');
    
    
    
    subplot(Np,3,(p-1)*3+3);
    hold on;
    plot(zeros(1,N),'k--');
    plot(ones(1,N),'k--');
    if p == 1
        thiscol = [1 0 0];
    else
        thiscol = [0 0 1];
    end
    plot(Cs{3, p},'Color', thiscol,'LineWidth',3);
    hold off;
    if p == 1
        title('singular values','FontSize',fs);
    end
    set(gca,'YLim',[-0.1,1.1], 'YTick',[0 1], 'FontSize',fs, 'Box','on');
    if p < 2
        set(gca,'XTick',[]);
    end
end

%%

%%% the 3D figure

% this uses a lot of Matlab surface rendering fine-tuning. Consult Matlab
% documentation for detail. 

k = 40; % control parameter for surface rendering
[S1 S2 S3] = sphere(k);
Spoints = zeros(3,n);

counter = 0;
for i = 1:k+1
    for j = 1:k+1
        counter = counter + 1;
        Spoints(:,counter) = [S1(i,j), S2(i,j), S3(i,j)]';
    end
end

Apoints = Cs{1, 1} * Spoints;
Bpoints = Cs{1, 2} * Spoints;
A1 = zeros(k+1,k+1);  A2 = zeros(k+1,k+1);  A3 = zeros(k+1,k+1);
B1 = zeros(k+1,k+1);  B2 = zeros(k+1,k+1);  B3 = zeros(k+1,k+1);
S1 = zeros(k+1,k+1);  S2 = zeros(k+1,k+1);  S3 = zeros(k+1,k+1);

counter = 0;
for i = 1:k+1
    for j = 1:k+1
        counter = counter + 1;
        A1(i,j) = Apoints(1,counter);
        A2(i,j) = Apoints(2,counter);
        A3(i,j) = Apoints(3,counter);
        B1(i,j) = Bpoints(1,counter);
        B2(i,j) = Bpoints(2,counter);
        B3(i,j) = Bpoints(3,counter);
        S1(i,j) = Spoints(1,counter);
        S2(i,j) = Spoints(2,counter);
        S3(i,j) = Spoints(3,counter);
    end
end

L = 100; % how many state points are plotted
trace1 =  allTrainArgs(:,1:L);
trace2 =  allTrainArgs(:,learnLength+1:learnLength+L);

alphaValue = 40;
lightpos = 5*[-1 2 1];
specexp = 6;
specstrength = 4;
ambstrength = .85;
diffstrength = 1;

figure(2); clf;
set(gcf, 'WindowStyle','normal');
set(gcf,'Position', [1200 200 400 400]);
fs = 14; % font size
rg = 1.1; % drawing box plusminus axis range
pointN = 100; % how many points are plotted per pattern
hold on;
% plot state dots
plot3(trace1(1,1:pointN), trace1(2,1:pointN), trace1(3,1:pointN), ...
    'r.', 'MarkerSize', 15);
plot3(trace2(1,1:pointN), trace2(2,1:pointN), trace2(3,1:pointN),...
    'b.','MarkerSize', 20);

% render unit sphere
surf(S1, S2, S3, 'FaceColor', 0.8*[1 1 1], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', 0.8, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', 10 * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', .5, ...
    'SpecularExponent', 3, ...
    'SpecularStrength', 1, 'DiffuseStrength', .5);

% render first ellipsoid
surf(A1, A2, A3, 'FaceColor', [1 .5 .5], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', ambstrength, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', alphaValue * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', 1, ...
    'SpecularExponent', specexp, ...
    'SpecularStrength', specstrength, 'DiffuseStrength', diffstrength);

% render second ellipsoid
surf(B1, B2, B3, 'FaceColor', [.5 .5 1], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', ambstrength, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', alphaValue * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', 1, ...
    'SpecularExponent', specexp, ...
    'SpecularStrength', specstrength, 'DiffuseStrength', diffstrength);


light('Position',lightpos,'Style','infinite');

% plot ellipsoid axes
U1 = Cs{2, 1}; U2 = Cs{2, 2}; Sig1 = Cs{3, 1}; Sig2 = Cs{3, 2};
line(Sig1(1) * [0,U1(1,1)], Sig1(1) * [0,U1(2,1)], ...
    Sig1(1) * [0,U1(3,1)],...
    'Color', 0.4*[1 0 0], 'LineWidth', 2);
line(Sig1(2) * [0,-U1(1,2)], Sig1(2) * [0,-U1(2,2)], ...
    Sig1(2) * [0,-U1(3,2)],...
    'Color', 0.4*[1 0 0], 'LineWidth', 2);
line(Sig1(3) * [0,U1(1,3)], Sig1(3) * [0,U1(2,3)], ...
    Sig1(3) * [0,U1(3,3)],...
    'Color', 0.4*[1 0 0], 'LineWidth', 2);
line(Sig2(1) * [0,-U2(1,1)], Sig2(1) * [0,-U2(2,1)], ...
    Sig2(1) * [0,-U2(3,1)],...
    'Color', 0.4*[ 0 0 1], 'LineWidth', 2);
line(Sig2(2) * [0,-U2(1,2)], Sig2(2) * [0,-U2(2,2)], ...
    Sig2(2) * [0,-U2(3,2)],...
    'Color', 0.4*[ 0 0 1], 'LineWidth', 2);

hold off;

set(gca, 'XLim', [-rg rg], 'YLim', [-rg rg],'ZLim', [-rg rg],...
    'XTick',[-1 0 1], 'YTick',[-1 0 1],'ZTick',[-1 0 1], ...
    'FontSize', fs, 'Box', 'on', 'CameraPosition', [11.61 14.7 3.4], ...
    'CameraTarget', [0 0 0]);
axis square;


%%% same, with 1/3 and 3-fold aperture
%%
figure(3); clf;
set(gcf, 'WindowStyle','normal');
set(gcf,'Position', [900 200 200 400]);
subplot(2,1,1);

R = patternRs{1}; C1 = R * inv(R + (alphas(1)/2)^(-2)*I);
R = patternRs{2}; C2 = R * inv(R + (alphas(2)/2)^(-2)*I);

Apoints = C1 * Spoints;
Bpoints = C2 * Spoints;
A1 = zeros(k+1,k+1);  A2 = zeros(k+1,k+1);  A3 = zeros(k+1,k+1);
B1 = zeros(k+1,k+1);  B2 = zeros(k+1,k+1);  B3 = zeros(k+1,k+1);
S1 = zeros(k+1,k+1);  S2 = zeros(k+1,k+1);  S3 = zeros(k+1,k+1);

counter = 0;
for i = 1:k+1
    for j = 1:k+1
        counter = counter + 1;
        A1(i,j) = Apoints(1,counter);
        A2(i,j) = Apoints(2,counter);
        A3(i,j) = Apoints(3,counter);
        B1(i,j) = Bpoints(1,counter);
        B2(i,j) = Bpoints(2,counter);
        B3(i,j) = Bpoints(3,counter);        
    end
end


hold on;
% plot state dots
plot3(trace1(1,1:pointN), trace1(2,1:pointN), trace1(3,1:pointN), ...
    'r.', 'MarkerSize', 5);
plot3(trace2(1,1:pointN), trace2(2,1:pointN), trace2(3,1:pointN),...
    'b.','MarkerSize', 10);

% render first ellipsoid
surf(A1, A2, A3, 'FaceColor', [1 .5 .5], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', ambstrength, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', alphaValue * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', 1, ...
    'SpecularExponent', specexp, ...
    'SpecularStrength', specstrength, 'DiffuseStrength', diffstrength);

% render second ellipsoid
surf(B1, B2, B3, 'FaceColor', [.5 .5 1], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', ambstrength, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', alphaValue * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', 1, ...
    'SpecularExponent', specexp, ...
    'SpecularStrength', specstrength, 'DiffuseStrength', diffstrength);

light('Position',lightpos,'Style','infinite');

hold off;

set(gca, 'XLim', [-rg rg], 'YLim', [-rg rg],'ZLim', [-rg rg],...
    'XTick',[], 'YTick',[],'ZTick',[], ...
    'FontSize', fs, 'Box', 'on', 'CameraPosition', [11.61 14.7 3.4], ...
    'CameraTarget', [0 0 0]);
axis square;

%%% same, with 3* aperture

R = patternRs{1}; C1 = R * inv(R + (2*alphas(1))^(-2)*I);
R = patternRs{2}; C2 = R * inv(R + (2*alphas(2))^(-2)*I);

Apoints = C1 * Spoints;
Bpoints = C2 * Spoints;
A1 = zeros(k+1,k+1);  A2 = zeros(k+1,k+1);  A3 = zeros(k+1,k+1);
B1 = zeros(k+1,k+1);  B2 = zeros(k+1,k+1);  B3 = zeros(k+1,k+1);
S1 = zeros(k+1,k+1);  S2 = zeros(k+1,k+1);  S3 = zeros(k+1,k+1);

counter = 0;
for i = 1:k+1
    for j = 1:k+1
        counter = counter + 1;
        A1(i,j) = Apoints(1,counter);
        A2(i,j) = Apoints(2,counter);
        A3(i,j) = Apoints(3,counter);
        B1(i,j) = Bpoints(1,counter);
        B2(i,j) = Bpoints(2,counter);
        B3(i,j) = Bpoints(3,counter);        
    end
end

subplot(2,1,2);

hold on;
% plot state dots
plot3(trace1(1,1:pointN), trace1(2,1:pointN), trace1(3,1:pointN), ...
    'r.', 'MarkerSize', 5);
plot3(trace2(1,1:pointN), trace2(2,1:pointN), trace2(3,1:pointN),...
    'b.','MarkerSize', 10);


% render first ellipsoid
surf(A1, A2, A3, 'FaceColor', [1 .5 .5], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', ambstrength, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', alphaValue * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', 1, ...
    'SpecularExponent', specexp, ...
    'SpecularStrength', specstrength, 'DiffuseStrength', diffstrength);

% render second ellipsoid
surf(B1, B2, B3, 'FaceColor', [.5 .5 1], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', ambstrength, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', alphaValue * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', 1, ...
    'SpecularExponent', specexp, ...
    'SpecularStrength', specstrength, 'DiffuseStrength', diffstrength);


light('Position',lightpos,'Style','infinite');

hold off;

set(gca, 'XLim', [-rg rg], 'YLim', [-rg rg],'ZLim', [-rg rg],...
    'XTick',[], 'YTick',[],'ZTick',[], ...
    'FontSize', fs, 'Box', 'on', 'CameraPosition', [11.61 14.7 3.4], ...
    'CameraTarget', [0 0 0]);
axis square;


%%% state contraction under first conceptor
%%

figure(4); clf;
set(gcf, 'WindowStyle','normal');
set(gcf,'Position', [800 200 800 400]);
fs = 14; % font size

Apoints = Cs{1, 1} * Spoints;
Bpoints = Cs{1, 2} * Spoints;
A1 = zeros(k+1,k+1);  A2 = zeros(k+1,k+1);  A3 = zeros(k+1,k+1);
B1 = zeros(k+1,k+1);  B2 = zeros(k+1,k+1);  B3 = zeros(k+1,k+1);
S1 = zeros(k+1,k+1);  S2 = zeros(k+1,k+1);  S3 = zeros(k+1,k+1);

counter = 0;
for i = 1:k+1
    for j = 1:k+1
        counter = counter + 1;
        A1(i,j) = Apoints(1,counter);
        A2(i,j) = Apoints(2,counter);
        A3(i,j) = Apoints(3,counter);
        B1(i,j) = Bpoints(1,counter);
        B2(i,j) = Bpoints(2,counter);
        B3(i,j) = Bpoints(3,counter);
        S1(i,j) = Spoints(1,counter);
        S2(i,j) = Spoints(2,counter);
        S3(i,j) = Spoints(3,counter);
    end
end

L = 100; % how many state points are plotted
trace1 =  allTrainArgs(:,1:L);
trace2 =  allTrainArgs(:,learnLength+1:learnLength+L);
trace1C1 = Cs{1, 1} * trace1;
trace2C1 = Cs{1, 1} * trace2;
trace1C2 = Cs{1, 2} * trace1;
trace2C2 = Cs{1, 2} * trace2;

alphaValue = 40;
lightpos = 5*[-1 2 1];
specexp = 6;
specstrength = 4;
ambstrength = .85;
diffstrength = 1;

subplot(1,2,1);

hold on;
% plot state dots
plot3(trace1(1,1:pointN), trace1(2,1:pointN), trace1(3,1:pointN), ...
    'r.', 'MarkerSize', 15);
plot3(trace2(1,1:2), trace2(2,1:2), trace2(3,1:2),...
    'b.','MarkerSize', 20);

% plot projected dots
plot3(trace1C1(1,1:pointN), trace1C1(2,1:pointN), trace1C1(3,1:pointN), ...
    '.', 'Color', 'k','MarkerSize', 15);
plot3(trace2C1(1,1:2), trace2C1(2,1:2), trace2C1(3,1:2),...
    '.', 'Color', 'k','MarkerSize', 20);

% render first ellipsoid
surf(A1, A2, A3, 'FaceColor', [1 .5 .5], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', ambstrength, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', alphaValue * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', 1, ...
    'SpecularExponent', specexp, ...
    'SpecularStrength', specstrength, 'DiffuseStrength', diffstrength);

light('Position',lightpos,'Style','infinite');
hold off;

set(gca, 'XLim', [-rg rg], 'YLim', [-rg rg],'ZLim', [-rg rg],...
    'XTick',[-1 0 1], 'YTick',[-1 0 1],'ZTick',[-1 0 1], ...
    'FontSize', fs, 'Box', 'on', 'CameraPosition', [11.61 14.7 3.4], ...
    'CameraTarget', [0 0 0]);
axis square;

subplot(1,2,2);

hold on;
% plot state dots
plot3(trace1(1,1:pointN), trace1(2,1:pointN), trace1(3,1:pointN), ...
    'r.', 'MarkerSize', 15);
plot3(trace2(1,1:2), trace2(2,1:2), trace2(3,1:2),...
    'b.','MarkerSize', 20);

% plot projected dots
plot3(trace1C2(1,1:pointN), trace1C2(2,1:pointN), trace1C2(3,1:pointN), ...
    '.', 'Color', 'k','MarkerSize', 15);
plot3(trace2C2(1,1:2), trace2C2(2,1:2), trace2C2(3,1:2),...
    '.', 'Color', 'k','MarkerSize', 20);

% render second ellipsoid
surf(B1, B2, B3, 'FaceColor', [.5 .5 1], 'EdgeColor', 'none', ...
    'FaceLighting', 'gouraud', 'AmbientStrength', ambstrength, ...
    'FaceAlpha', 'flat', ...
    'AlphaDataMapping','direct',...
    'AlphaData', alphaValue * ones(k+1,k+1), ...
    'BackFaceLighting', 'unlit', ...
    'SpecularColorReflectance', 1, ...
    'SpecularExponent', specexp, ...
    'SpecularStrength', specstrength, 'DiffuseStrength', diffstrength);


light('Position',lightpos,'Style','infinite');
hold off;

set(gca, 'XLim', [-rg rg], 'YLim', [-rg rg],'ZLim', [-rg rg],...
    'XTick',[-1 0 1], 'YTick',[-1 0 1],'ZTick',[-1 0 1], ...
    'FontSize', fs, 'Box', 'on', 'CameraPosition', [11.61 14.7 3.4], ...
    'CameraTarget', [0 0 0]);
axis square;

% %%% morphing
% %%
% morphPureLength = 25;
% morphLength = 50;
% morphFactors =  (0:morphLength-1) / (morphLength-1);
% 
% morphcollector = zeros(1, 2*morphPureLength + morphLength);
% 
% R = patternRs{1}; C1 = R * inv(R + (3.3*alphas(1))^(-2)*I);
% R = patternRs{2}; C2 = R * inv(R + (3*alphas(2))^(-2)*I);
% 
% x = 0.5 * randn(N,1);
% for n = 1:washoutLength
%     x = C1 * tanh(W * x + Wbias);
% end
% for n = 1:morphPureLength
%     x = C1 * tanh(W * x + Wbias);
%     morphcollector(1,n) = Wout * x;
% end
% 
% for n = 1:morphLength
%     x = ((1-morphFactors(n)) * C1 + ...
%         morphFactors(n) * C2) * tanh(W * x + Wbias);
%     morphcollector(1, morphPureLength + n) = Wout * x;
% end
% for n = 1:morphPureLength
%     x = C2 * tanh(W * x + Wbias);
%     morphcollector(1, morphPureLength + morphLength + n) = Wout * x;
% end
% 
% figure(5); clf;
% set(gcf, 'WindowStyle','normal');
% set(gcf,'Position', [800 500 800 200]);  
% plot(morphcollector);

