%%%% Demo: morphing three patterns from a 2-parametric pattern family.
%%%% H. Jaeger, May 18, 2015
%%%% Designed using Matlab R2008b
%%%%
%%%% Note: users running later versions of Matlab reported that some
%%%% conceptor demos wouldn't succeed. The cause seems to be that my old
%%%% Matlab version uses a different (and better) implementation of the inv
%%%% function than later Matlab versions. Replacing all calls to "inv" by
%%%% calls to "pinv" so far has always resolved the problems. 

addpath('./helpers');

%%% Experiment control
randstate = 1; newNets = 1; newSystemScalings = 1;

%%% Setting system params
N = 100; % network size
NetSR = 2.2; % spectral radius
NetinpScaling = NetSR; % scaling of input weights
BiasScaling = 1.5; % size of bias


%%% loading learning
TychonovAlpha = .001; % ridge regression regularizer for  W training
washoutLength = 500;
learnLength = 1000;

%%% pattern readout learning
TychonovAlphaReadout = 0.0001;


%%% C learning and testing
alpha = 2;
CtestLength = 200;
SplotLength = 50;

%%% morphing
morphTime = 100; morphWashout = 50;


%%% plotting
showDotplot = 0; % whether to plot delay embedding plots
plotTargetPatterns = 1; % whether to plot the 3 target patterns in separate plot
granularity = 'coarse'; % options are 'coarse' and 'fine'
borderPanelN = 8; % how many lines of panels surround the center triangle
morphPlotPoints = 20; % how many time points used for pattern plots
dotPlotDelay = 2; % delay for time embedding for delay embedding plot
morphDotPlotPoints = 100; % how many points are plotted in delay embedding plot
                          % Must be at most morphTime


%%% Defining 3 target patterns

P = 8.212113; % reference period length
FreqFactor = 1.1; % factor by which freq is changed when going left
                  % in the diagram by half the length of the bottom segment
                  % of the triangle
ShapeFactor = .2; % when going up in diagram by 1/3 the hight of the
                  % triangle, previous pattern (normalized to range 0-1) is raised
                  % power exp(ShapeFactor)
patterns = [1 2 3]; % order in which 3 patterns figure in the triangle
a = - sqrt(3) * FreqFactor; b = -1 * ShapeFactor; 
patts{1} = @(n) 2*((0.5 * sin(2 * pi * n / (P + a)) + 0.5).^exp(b)) - 1;
a = sqrt(3)  * FreqFactor; b = -1 * ShapeFactor;
patts{2} = @(n) 2*((0.5 * sin(2 * pi * n / (P + a)) + 0.5).^exp(b)) - 1;
a = 0; b = 2 * ShapeFactor;
patts{3} = @(n) 2*((0.5 * sin(2 * pi * n / (P + a)) + 0.5).^exp(b)) - 1;



%%% Initializations

randn('state', randstate);
rand('twister', randstate);

% Create raw weights
if newNets
    if N <= 20
        Netconnectivity = 1;
    else
        Netconnectivity = 10/N;
    end
    WstarRaw = generate_internal_weights(N, Netconnectivity);
    WinRaw = randn(N, 1);
    WbiasRaw = randn(N, 1);
end

% Scale raw weights and initialize weights
if newSystemScalings
    Wstar = NetSR * WstarRaw;
    Win = NetinpScaling * WinRaw;
    Wbias = BiasScaling * WbiasRaw;
end


I = eye(N);

%%% load patterns and compute output weights

% initialize data collectors
Np = length(patterns); % number of patterns (here always Np = 3)
allTrainArgs = zeros(N, Np * learnLength);
allTrainOldArgs = zeros(N, Np * learnLength);
allTrainOuts = zeros(1, Np * learnLength);
patternRs = cell(1,Np);

% collect data from driving native reservoir with different drivers
for p = 1:Np
    patt = patts{patterns(p)}; % current pattern generator
    xCollector = zeros(N, learnLength );
    xOldCollector = zeros(N, learnLength );
    pCollector = zeros(1, learnLength );
    x = zeros(N, 1);
    for n = 1:(washoutLength + learnLength)
        u = patt(n); % pattern input
        xOld = x;
        x = tanh(Wstar * x + Win * u + Wbias);
        if n > washoutLength
            xCollector(:, n - washoutLength ) = x;
            xOldCollector(:, n - washoutLength ) = xOld;
            pCollector(1, n - washoutLength) = u;
        end
    end   
    
    R = xCollector * xCollector' / learnLength;
    patternRs{p} = R;        
    
    allTrainArgs(:, (p-1)*learnLength+1:p*learnLength) = ...
        xCollector;
    allTrainOldArgs(:, (p-1)*learnLength+1:p*learnLength) = ...
        xOldCollector;
    allTrainOuts(1, (p-1)*learnLength+1:p*learnLength) = ...
        pCollector;   
end

%%% compute readout
L = Np * learnLength;
Wout = (inv(allTrainArgs * allTrainArgs' / L + ...
    TychonovAlphaReadout * eye(N)) * allTrainArgs * allTrainOuts' / L)';
% training error
NRMSE_readout = nrmse(Wout*allTrainArgs, allTrainOuts);
disp(sprintf('Wout NRMSE: %g  absWeight: %g', ...
    NRMSE_readout, mean(abs(Wout))));

%%% compute W
Wtargets = (atanh(allTrainArgs) - repmat(Wbias,1,Np*learnLength));
W = (inv(allTrainOldArgs * allTrainOldArgs' / L + ...
    TychonovAlpha * eye(N)) * allTrainOldArgs * Wtargets' / L)';
% training errors per neuron
NRMSE_W = nrmse(W*allTrainOldArgs, Wtargets);
disp(sprintf('W NRMSE: %g  absWeight: %g', ...
    mean(NRMSE_W), mean(mean(abs(W)))));


%%% compute conceptors
Cs = cell(4, Np);
for p = 1:Np
    R = patternRs{p};
    [U S V] = svd(R);
    Snew = (S * inv(S + alpha^(-2) * eye(N)));
    
    C = U * Snew * U';
    Cs{1, p} = C;
    Cs{2, p} = U;
    Cs{3, p} = diag(Snew);
    Cs{4, p} = diag(S);
end

%%% Coordinate transform helpers to negotiate between the mathematical
%%% parameter space (the x,y plane where x, y correspond to the two pattern
%%% parameters period length and sinewave exponent, such that the origin
%%% has reference period length P and unit (none) exponentiation, i.e. pure 
%%% sinewave) and the plotting area with coordinates [0 0] for left bottom 
%%% [1 1 ] right top. 

% M is the linear transformation from [x y 1]' to mixture vectors for the
% three reference pattern. It maps the left bottom of the triangle to [1 0
% 0]', the right bottom to [0 1 0], the top to [0 0 1]
M = [- 1/(2*sqrt(3)) -1/6 1/3;...
    1/(2*sqrt(3)) -1/6 1/3;...
    0 1/3 1/3];

% set nr of panels and panel location increments for the two panel
% resolutions
if strcmp(granularity, 'fine')
    tN = 7 + 2*borderPanelN;
    xinc = 1/sqrt(3); yinc = 1/2;
elseif strcmp(granularity, 'coarse')
    tN = 5 + 2*borderPanelN;
    xinc = sqrt(3)/2; yinc = 3/4;
end

% compute mixture vectors
mixVecs = cell(tN, tN);
for i = 1:tN
    for j = 1:tN
        % compute x, y coordinates corresponding to midpoint of panel
        if strcmp(granularity, 'fine')
            x = -(3+borderPanelN)/sqrt(3) + (j-1)*xinc;
            y = -(1+0.5*borderPanelN) + (i-1)*yinc;
        elseif strcmp(granularity, 'coarse')
            x = - xinc * (2+borderPanelN) + (j-1)*xinc;
            y = - yinc * (4/3+borderPanelN) + (i-1)*yinc;
        end
        mixVecs{i,j} = M * [x y 1]';
    end
end

% compute morphed patterns
C1 = Cs{1,1}; C2 = Cs{1,2}; C3 = Cs{1,3};
x0 = rand(N,1); % starting reservoir state
plots = cell(tN, tN);
for i = 1:tN
    for j = 1:tN
        mixVec = mixVecs{i,j};
        Cmix = mixVec(1)*C1 + mixVec(2)*C2 + mixVec(3)*C3 ;
        
        yColl = zeros(1,morphTime); 
        x = x0;
        % washout
        for n = 1:morphWashout
            x = Cmix * tanh(W * x + Wbias);
        end
        % collect x
        for n = 1:morphTime
            x = Cmix * tanh(W * x + Wbias);
            yColl(1,n) = Wout * x;
        end
        plots{i,j} = yColl;
    end
    
end


%%% Plotting

% the three reference patterns in close-up
if plotTargetPatterns
    figure(1); clf;
    set(gcf, 'WindowStyle','normal');
    set(gcf,'Position', [900 1000 600 150]);
    plotLength = morphPlotPoints; fs = 14;
    for i = 1:3
        subplot(1,3,i);
        patt = patts{patterns(i)};
        plot(patt(1:plotLength), 'k', 'LineWidth', 3);
        set(gca, 'XLim', [1, plotLength], 'YLim', [-1.2 1.2], 'Fontsize', fs);
        if i == 1
            set(gca, 'Color', 'r');
        elseif i == 2
            set(gca, 'Color', 'g');
        elseif i == 3
            set(gca, 'Color', 'b');
        end
    end    
end

%% the main figure with all those panels
c = 0.85; % coloring transparency for backgrounds, between 0 and 1
col1 = [1 c c]; col2 = [c 1 c]; col3 = [c c 1]; % set reference colors
figure(2); clf;
set(gcf, 'WindowStyle','normal');
plotwidth = 800;
if strcmp(granularity, 'fine')
plothight = plotwidth * yinc / xinc;
elseif strcmp(granularity, 'coarse')
    plothight = plotwidth * yinc / xinc;
end
set(gcf,'Position', [850 100 plotwidth plothight]);
panelWidth = 1/(tN + 1); % in normalized plot window units
panelHight = 1/(tN + 1);
for i = 1:tN
    for j = 1:tN
        panelx = (j-1)*(1/tN);
        panely = (i-1)*(1/tN);
        subplot('Position', [panelx, panely, panelWidth, panelHight]);
        thisdata = plots{i,j}(1,1:morphPlotPoints);
        
        
        plot(thisdata, 'k', 'LineWidth',1.5);
        thisColor = mixVecs{i,j}' * [col1; col2; col3];
        thisColor = min(max(thisColor, 0), 1);
        
        set(gca, 'XTick',[],'YTick',[],...
            'YLim', [-1.2,1.2], 'XLim',[1 morphPlotPoints], 'Color', ...
            thisColor, 'Box', 'off');
        
        if strcmp(granularity, 'fine')
            if (i == borderPanelN + 1 && j == borderPanelN + 1)
                set(gca, 'Color', [1 0 0]);
            elseif (i == borderPanelN + 1 && j == borderPanelN + 7)
                set(gca, 'Color', [0 1 0]);
            elseif (i == borderPanelN + 7 && j == borderPanelN + 4)
                set(gca, 'Color', [0  0 1]);
            end
        elseif strcmp(granularity, 'coarse')
            if (i == borderPanelN + 1 && j == borderPanelN + 1)
                set(gca, 'Color', [1 0 0]);
            elseif (i == borderPanelN + 1 && j == borderPanelN + 5)
                set(gca, 'Color', [0 1 0]);
            elseif (i == borderPanelN + 5 && j == borderPanelN + 3)
                set(gca, 'Color', [0  0 1]);
            end
        end
        
    end
end

% similar, for the delay embedding plots
if showDotplot
    figure(3); clf;
    set(gcf, 'WindowStyle','normal');
    set(gcf,'Position', [850 150 plotwidth plothight]);
    panelWidth = 1/(tN + 1);
    panelHight = 1/(tN + 1);
    for i = 1:tN
        for j = 1:tN
            panelx = (j-1)*(1/tN);
            panely = (i-1)*(1/tN);
            subplot('Position', [panelx, panely, panelWidth, panelHight]);
            thisdata = plots{i,j}(1,1:morphDotPlotPoints);
            
            delayPL = tanh([thisdata(1,1:end-dotPlotDelay); ...
                thisdata(1, (dotPlotDelay+1):end)]);
            
            plot(delayPL(1,:), delayPL(2,:), 'k.', 'LineWidth',3);
            thisColor = mixVecs{i,j}' * [col1; col2; col3];
            thisColor = min(max(thisColor, 0), 1);
            
            set(gca, 'XTick',[],'YTick',[],...
                'YLim', [-1,1], 'XLim',[-1 1], 'Color', ...
                thisColor);
            
            if strcmp(granularity, 'fine')
                if (i == borderPanelN + 1 && j == borderPanelN + 1)
                    set(gca, 'Color', [1 0 0]);
                elseif (i == borderPanelN + 1 && j == borderPanelN + 7)
                    set(gca, 'Color', [0 1 0]);
                elseif (i == borderPanelN + 7 && j == borderPanelN + 4)
                    set(gca, 'Color', [0  0 1]);
                end
            elseif strcmp(granularity, 'coarse')
                if (i == borderPanelN + 1 && j == borderPanelN + 1)
                    set(gca, 'Color', [1 0 0]);
                elseif (i == borderPanelN + 1 && j == borderPanelN + 5)
                    set(gca, 'Color', [0 1 0]);
                elseif (i == borderPanelN + 5 && j == borderPanelN + 3)
                    set(gca, 'Color', [0  0 1]);
                end
            end
        end
    end
end



