%%%% demo: content-addressable memory with the aid of autoconceptors; here:
%%%%       class learning effect
%%%% H. Jaeger June 14, 2015
%%%% Designed using Matlab R2008b
%%%%
%%%% Note: I use suffix "PL" for "PlotList" in my telling-naming of
%%%% variables
%%%%
%%%% Note: users running later versions of Matlab reported that some
%%%% conceptor demos wouldn't succeed. The cause seems to be that my old
%%%% Matlab version uses a different (and better) implementation of the inv
%%%% function than later Matlab versions. Replacing all calls to "inv" by
%%%% calls to "pinv" so far has always resolved the problems.

addpath('./helpers');

%%% Experiment control
randstate = 1; % determines seed for random nr generation
expN = 10; % Nr of experiments over which to average

%%% Setting system params
N = 100; % network size
SR = 1.1; % spectral radius of reservoir weight matrix
WinScaling = 1.1; % scaling of pattern feeding weights
biasScaling = .25;

%%% Loading
trainWashoutLength = 100;
learnWLength = 500;
TychonovAlphaD = .01; % ridge regularizer for D
TychonovAlphaWout = 0.01; % ridge regularizer for Wout

%%% C adaptation and testing
aperture = 1000;
initialWashout = 100;
cueLength = 12;
cueNL = 0.0;

CadaptRateCue = 0.01;
CadaptRateAfterCue = 0.01; % C adaptation rate
SNRTest = 1 ; % state signal-to-noise ratio during auto-adaptation
SNRCue = Inf; % state signal-to-noise ratio during cue. Set to Inf if
% no noise is wanted
RL = 10000; % auto-adaptation runlength after cue

%%% computing diagnostics
measureWashout = 50;
measureTemplateLength = 20;
measureRL = 500; % runlength for measuring output NRMSE.
% Must be at least twice measureTemplateLength
signalPlotLength = 15;
singValPlotLength = 20;

%%% Setting patterns
NpLoads = [1 2 3 5 10 30 100 300 1000];
NpTest = 10; % number of patterns to be tested

%%% Initializations
randn('state', randstate);
rand('twister', randstate);
I = eye(N);

%%% main experiment repetition loop
allNRMSEOwns = cell(length(NpLoads), expN);
allNRMSEOthers = cell(length(NpLoads), expN);
for expi = 1:expN
    disp(sprintf('******* experiment Nr %g ********', expi));
    
    % Create raw weights
    WRaw = generate_internal_weights(N, 10/N);
    WinRaw = randn(N, 1);
    biasRaw = randn(N, 1);
    
    % Scale raw weights and initialize trainable weights
    W = SR * WRaw;
    Win = WinScaling * WinRaw;
    bias = biasScaling * biasRaw;
    
    % Set pattern handles
    NpLoadMax = NpLoads(end);
    patts = cell(1,NpLoadMax);
    pattsOther = cell(1,NpTest);
    stretch = sqrt(30);
    for p = 1:NpLoadMax
        relativePhase = rand;
        firstAmp = rand;
        patts{p} = @(n) ...
            firstAmp * sin(2 * pi * n / stretch) + ...
            (1 - firstAmp) * ...
            sin(4 * pi * (n / stretch + relativePhase));
    end
    for p = 1:NpTest
        relativePhase = rand;
        firstAmp = rand;
        pattsOther{p} = @(n) ...
            firstAmp * sin(2 * pi * n / stretch) + ...
            (1 - firstAmp) * ...
            sin(4 * pi * (n / stretch + relativePhase));
    end
    
    % pattern loading
    loadIndex = 0;
    
    for NpLoad = NpLoads
        disp(sprintf('NrLoad %g', NpLoad));
        loadIndex = loadIndex + 1;
        LL = NpLoad * learnWLength;
        allTrainArgs = zeros(N, N);
        pTemplates = zeros(measureTemplateLength, NpLoad);
        allTrainOldArgs = zeros(N, N);
        allTrainOldArgsDTargs = zeros(N, N);
        allTrainArgsYTargs = zeros(N,1);
        
        for p = 1:NpLoad
            XCue = zeros(N,learnWLength);
            XOldCue = zeros(N,learnWLength);
            pTemplate = zeros(learnWLength,1);
            patt = patts{p};
            signValCollectCounter = 1;
            x = zeros(N, 1);
            for n = 1:trainWashoutLength
                u = patt(n);
                x =  tanh(W * x + Win * u + bias);
            end
            for n = 1: learnWLength
                u = patt(n + trainWashoutLength);
                XOldCue(:,n) = x;
                x = tanh(W * x + Win * u + bias);
                XCue(:,n) = x;
                pTemplate(n,1) = u;
            end
            pTemplates(:,p) = pTemplate(end-measureTemplateLength+1:end);
            allTrainArgs = allTrainArgs + XCue * XCue';
            allTrainOldArgs = allTrainOldArgs + XOldCue * XOldCue';
            allTrainOldArgsDTargs = allTrainOldArgsDTargs + ...
                XOldCue * (Win * pTemplate')';
            allTrainArgsYTargs = allTrainArgsYTargs + ...
                XCue * pTemplate;
        end
        
        % compute mean variance of x (needed for setting the noise SNR)
        varx = mean(var(XCue'));
        
        % learn D
        D = (inv(allTrainOldArgs / LL + TychonovAlphaD * I) ...
            * allTrainOldArgsDTargs / LL )';
        
        %  learn readouts
        Wout = (inv(allTrainArgs' / LL + TychonovAlphaWout * I) ...
            * allTrainArgsYTargs  / LL)';
        
        %%% test retrieval with training patterns (maximally NpTest many)
        
        NpTestEff = min(NpLoad, NpTest);
        yTest_PL = zeros(measureRL, NpTestEff, 2);
        for p = 1:NpTestEff
            patt = patts{p};
            x = zeros(N,1);
            % initial C adaptation from cue run
            for n = 1:initialWashout
                u = patt(n);
                x =  tanh(W * x + Win * u + bias);
            end
            C = zeros(N,N);
            b = sqrt(varx /  SNRCue);
            for n = 1:cueLength
                u = patt(n + initialWashout) + cueNL * randn;
                x =  tanh(W * x + Win * u + bias ) + b * randn(N,1);
                C = C + CadaptRateCue * ((x - C*x)*x' - aperture^(-2)*C);
            end
            
            
            % measure quality after cue
            xBeforeMeasure = x;
            for n = 1:measureWashout
                x = C * tanh(W *  x + D * x + bias);
            end
            
            for n = 1:measureRL
                r = tanh(W *  x + D * x + bias);
                x = C * r;
                yTest_PL(n,p,1) = Wout * x;
            end
            x = xBeforeMeasure;
            
            % continue run with autoadaptation
            
            % state noise scaling factor b
            b = sqrt(varx /  SNRTest);
            for n = 1:RL
                x = C * (tanh(W *  x + D * x + bias )+ b * randn(N,1));
                C = C + CadaptRateAfterCue * ((x - C*x)*x' - aperture^(-2)*C);
            end
            
            for n = 1:measureWashout
                x = C * tanh(W *  x + D * x + bias);
            end
            for n = 1:measureRL
                r = tanh(W *  x + D * x + bias);
                x = C * r;
                yTest_PL(n,p,2) = Wout * x;
            end
            
        end
        
        %%
        % optimally align C-reconstructed readouts with drivers
        intRate = 10;
        pAligned_PL = zeros(NpTestEff, measureTemplateLength, 2);
        NRMSEsAlignedOwn = zeros(NpTestEff, 2);
        pTemplatesUsedInTest = pTemplates;
        
        for i = 1:2
            for p = 1:NpTestEff
                thisDriver = pTemplatesUsedInTest(:,p);
                thisOut = yTest_PL(:,p,i);
                thisDriverInt = interp1((1:measureTemplateLength)',thisDriver,...
                    (1:(1/intRate):measureTemplateLength)', 'spline')';
                thisOutInt = interp1((1:measureRL)', thisOut,...
                    (1:(1/intRate):measureRL)', 'spline')';
                
                L = size(thisOutInt,2); M = size(thisDriverInt,2);
                phasematches = zeros(1,L - M);
                for phaseshift = 1:(L - M)
                    phasematches(1,phaseshift) = ...
                        norm(thisDriverInt - ...
                        thisOutInt(1,phaseshift:phaseshift+M-1));
                end
                [maxVal maxInd] = max(-phasematches);
                pAligned_PL(p,:,i) = ...
                    thisOutInt(maxInd:intRate:...
                    (maxInd+intRate*measureTemplateLength-1))';
                
                NRMSEsAlignedOwn(p,i) = ...
                    nrmse(pAligned_PL(p,:,i),thisDriver');
            end
        end
        
        
        disp(sprintf('log10 mean NRMSEs test fromTrain %s',...
            num2str(log10(mean(NRMSEsAlignedOwn, 1)),2)));
        
        
        allNRMSEOwns{loadIndex, expi} = NRMSEsAlignedOwn;
        
        %%% test retrieval with other patterns
        
        NpTestEff = NpTest;
        yTest_PL = zeros(measureRL, NpTestEff, 2);
        for p = 1:NpTestEff
            patt = pattsOther{p};
            x = zeros(N,1);
            % initial preliminary C estimation from driven run
            for n = 1:initialWashout
                u = patt(n);
                x =  tanh(W * x + Win * u + bias);
            end
            C = zeros(N,N);
            b = sqrt(varx /  SNRCue);
            for n = 1:cueLength
                u = patt(n + initialWashout) + cueNL * randn;
                x =  tanh(W * x + Win * u + bias ) + b * randn(N,1);
                C = C + CadaptRateCue * ((x - C*x)*x' - aperture^(-2)*C);
            end
            
            
            % measure quality directly after cue
            xBeforeMeasure = x;
            for n = 1:measureWashout
                x = C * tanh(W *  x + D * x + bias);
            end
            for n = 1:measureRL
                x = C * tanh(W *  x + D * x + bias);
                yTest_PL(n,p,1) = Wout * x;
            end
            x = xBeforeMeasure;
            
            % run with autoadaptation
            
            % state noise scaling factor b
            b = sqrt(varx /  SNRTest);
            for n = 1:RL
                x = C * (tanh(W *  x + D * x + bias )+ b*randn(N,1));
                C = C + CadaptRateAfterCue * ((x - C*x)*x' - aperture^(-2)*C);
            end
            
            % measure quality at end of auto-adaptation
            for n = 1:measureWashout
                x = C * tanh(W *  x + D * x + bias);
            end
            for n = 1:measureRL
                x = C * tanh(W *  x + D * x + bias);
                yTest_PL(n,p,2) = Wout * x;
            end
        end
        
        
        % optimally align C-reconstructed readouts with drivers
        intRate = 10;
        pAligned_PL = zeros(NpTestEff, measureTemplateLength, 2);
        NRMSEsAlignedOther = zeros(NpTestEff, 2);
        pTemplatesUsedInTest = zeros(measureTemplateLength, NpTestEff);
        for p = 1:NpTestEff
            patt = pattsOther{p};
            pTemplatesUsedInTest(:,p) = patt(1:measureTemplateLength)';
        end
        
        for i = 1:2
            for p = 1:NpTestEff
                thisDriver = pTemplatesUsedInTest(:,p);
                thisOut = yTest_PL(:,p,i);
                thisDriverInt = interp1((1:measureTemplateLength)',thisDriver,...
                    (1:(1/intRate):measureTemplateLength)', 'spline')';
                thisOutInt = interp1((1:measureRL)', thisOut,...
                    (1:(1/intRate):measureRL)', 'spline')';
                
                L = size(thisOutInt,2); M = size(thisDriverInt,2);
                phasematches = zeros(1,L - M);
                for phaseshift = 1:(L - M)
                    phasematches(1,phaseshift) = ...
                        norm(thisDriverInt - ...
                        thisOutInt(1,phaseshift:phaseshift+M-1));
                end
                [maxVal maxInd] = max(-phasematches);
                pAligned_PL(p,:,i) = ...
                    thisOutInt(maxInd:intRate:...
                    (maxInd+intRate*measureTemplateLength-1))';
                
                NRMSEsAlignedOther(p,i) = ...
                    nrmse(pAligned_PL(p,:,i),thisDriver');
            end
        end
        
        disp(sprintf('log10 mean NRMSEs test fromOther %s',...
            num2str(log10(mean(NRMSEsAlignedOther)),2)));
        allNRMSEOthers{loadIndex, expi} = NRMSEsAlignedOther;
    end
end

%% plotting

% concatenate across experiments and take log10
allNRMSEconcatOwns = cell(1,length(NpLoads));
allNRMSEconcatOthers = cell(1,length(NpLoads));
for expi = 1:expN
    for loadi = 1:length(NpLoads)
        allNRMSEconcatOwns{1,loadi} = ...
            [allNRMSEconcatOwns{1,loadi}; ...
            log10(allNRMSEOwns{loadi, expi})];
        allNRMSEconcatOthers{1,loadi} = ...
            [allNRMSEconcatOthers{1,loadi}; ...
            log10(allNRMSEOthers{loadi, expi})];
    end
end

% compute Nrs of samples
NSampleOwn = zeros(1,length(NpLoads));
NSampleOther = zeros(1,length(NpLoads));
for loadi = 1:length(NpLoads)
    NSampleOwn(loadi) = size(allNRMSEconcatOwns{loadi},1);
    NSampleOther(loadi) = size(allNRMSEconcatOthers{loadi},1);
end

% compute means and variances
meanNRMSETestOwn = zeros(1,length(NpLoads));
meanNRMSETestOther = zeros(1,length(NpLoads));
for loadi = 1:length(NpLoads)
    meanNRMSETestOwn(1,loadi) = mean(allNRMSEconcatOwns{loadi}(:,2));
    meanNRMSETestOther(1,loadi) = mean(allNRMSEconcatOthers{loadi}(:,2));
end
varNRMSETestOwn = zeros(1,length(NpLoads));
varNRMSETestOther = zeros(1,length(NpLoads));
for loadi = 1:length(NpLoads)
    varNRMSETestOwn(1,loadi) = var(allNRMSEconcatOwns{loadi}(:,2));
    varNRMSETestOther(1,loadi) = var(allNRMSEconcatOthers{loadi}(:,2));
end


%%
figure(1); clf;
fs = 16; fstext = 16; % font sizes
set(gcf, 'WindowStyle','normal');
set(gcf,'Position', [1000 300  500 400]);
hold on;
errorbar(NpLoads, meanNRMSETestOther, ...
    1.96*sqrt(varNRMSETestOther) ./ sqrt(NSampleOther) , 'g');
errorbar(NpLoads, meanNRMSETestOwn, ...
    1.96*sqrt(varNRMSETestOwn) ./ sqrt(NSampleOwn), 'b');
plot(NpLoads, meanNRMSETestOther, 'g', 'LineWidth', 2);
plot(NpLoads, meanNRMSETestOwn, 'b', 'LineWidth', 2);
hold off;
set(gca, 'XLim', [0.8 NpLoads(end)+150], 'XTick', NpLoads, ...
    'XScale', 'log', 'YLim',[-1.21 -0.19], 'FontSize', fs);
xlabel('Nr of loaded patterns', 'FontSize',fstext);
ylabel('log10 NRMSE', 'FontSize',fstext);


%%
